@@ -9,8 +9,7 @@ void schlatherfull(int *covmod, double *data, double *dist, int *nSite, int *nOb
   //margins are unit Frechet, or the GEV parameters are estimated at
   //each locations.
   
-  const int nPairs = *nSite * (*nSite - 1) / 2,
-    nSitenObs = *nSite * *nObs;
+  const int nPairs = *nSite * (*nSite - 1) / 2;
   int i;
   double *jac, *rho, *frech;
   
@@ -27,6 +26,11 @@ void schlatherfull(int *covmod, double *data, double *dist, int *nSite, int *nOb
       }
     }
   }
+
+  if (*sill > 1){
+    *dns = *sill * *sill * MINF;
+    return;
+  }
    
   //Stage 0: Compute the covariance at each location
   switch (*covmod){
@@ -66,9 +70,8 @@ void schlatherfull(int *covmod, double *data, double *dist, int *nSite, int *nOb
   }
     
   else {
-    for (i=nSitenObs;i--;)
-      jac[i] = 0.0;
-    
+    memset(jac, 0, *nSite * *nObs * sizeof(double));
+        
     if (*weighted)
       *dns = wlplikschlather(data, rho, jac, *nObs, *nSite, weights);
 
@@ -80,20 +83,26 @@ void schlatherfull(int *covmod, double *data, double *dist, int *nSite, int *nOb
 
 }
 
-void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-		      int *dim, int *weighted, double *weights, double *locdsgnmat,
-		      double *locpenmat, int *nloccoeff, int *npparloc,
-		      double *locpenalty, double *scaledsgnmat, double *scalepenmat,
-		      int *nscalecoeff, int *npparscale, double *scalepenalty,
-		      double *shapedsgnmat, double *shapepenmat, int *nshapecoeff,
-		      int *npparshape, double *shapepenalty, double *loccoeff,
-		      double *scalecoeff, double *shapecoeff, double *sill,
-		      double *range, double *smooth, double *smooth2, double *dns){
+void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs, int *dim,
+		      int *weighted, double *weights, double *locdsgnmat, double *locpenmat,
+		      int *nloccoeff, int *npparloc, double *locpenalty, double *scaledsgnmat,
+		      double *scalepenmat, int *nscalecoeff, int *npparscale,
+		      double *scalepenalty, double *shapedsgnmat, double *shapepenmat,
+		      int *nshapecoeff, int *npparshape, double *shapepenalty, int *usetempcov,
+		      double *tempdsgnmatloc, double *temppenmatloc, int *ntempcoeffloc,
+		      int *nppartempcoeffloc, double *temppenaltyloc, double *tempdsgnmatscale,
+		      double *temppenmatscale, int *ntempcoeffscale, int *nppartempcoeffscale,
+		      double *temppenaltyscale, double *tempdsgnmatshape, double *temppenmatshape,
+		      int *ntempcoeffshape, int *nppartempcoeffshape, double *temppenaltyshape,
+		      double *loccoeff, double *scalecoeff, double *shapecoeff,
+		      double *tempcoeffloc, double *tempcoeffscale, double *tempcoeffshape,
+		      double *sill, double *range, double *smooth, double *smooth2, double *dns){
   //This is the Schlather's model.
   //The GEV parameters are defined using a polynomial response surface
   
   const int nPairs = *nSite * (*nSite - 1) / 2;
-  double *jac, *rho, *locs, *scales, *shapes, *frech;
+  int flag = usetempcov[0] + usetempcov[1] + usetempcov[2];
+  double *jac, *rho, *locs, *scales, *shapes, *frech, *trendlocs, *trendscales, *trendshapes;
     
   jac = (double *)R_alloc(*nObs * *nSite, sizeof(double));
   rho = (double *)R_alloc(nPairs, sizeof(double));
@@ -102,6 +111,11 @@ void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *
   shapes = (double *)R_alloc(*nSite, sizeof(double));
   frech = (double *)R_alloc(*nObs * *nSite, sizeof(double));
   
+  if (*sill > 1){
+    *dns = *sill * *sill * MINF;
+    return;
+  }
+
   //Stage 1: Compute the covariance at each location
   switch (*covmod){
   case 1:
@@ -125,17 +139,37 @@ void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *
     return;
 
   //Stage 2: Compute the GEV parameters using the design matrix
-  *dns = dsgnmat2Param(locdsgnmat, scaledsgnmat, shapedsgnmat,
-		       loccoeff, scalecoeff, shapecoeff, *nSite,
-		       *nloccoeff, *nscalecoeff, *nshapecoeff,
-		       locs, scales, shapes);
+  *dns = dsgnmat2Param(locdsgnmat, scaledsgnmat, shapedsgnmat, loccoeff, scalecoeff, shapecoeff,
+		       *nSite, *nloccoeff, *nscalecoeff, *nshapecoeff, locs, scales, shapes);
 
-  if (*dns != 0.0)
+  if (flag){
+    int i, j;
+    trendlocs = (double *)R_alloc(*nObs, sizeof(double));
+    trendscales = (double *)R_alloc(*nObs, sizeof(double));
+    trendshapes = (double *)R_alloc(*nObs, sizeof(double));
+
+    dsgnmat2temptrend(tempdsgnmatloc, tempdsgnmatscale, tempdsgnmatshape, tempcoeffloc,
+		      tempcoeffscale, tempcoeffshape, *nSite, *nObs, usetempcov, *ntempcoeffloc,
+		      *ntempcoeffscale, *ntempcoeffshape, trendlocs, trendscales, trendshapes);
+
+    for (i=*nSite;i--;)
+      for (j=*nObs;j--;)
+	if (((scales[i] + trendscales[j]) <= 0) || ((shapes[i] + trendshapes[j]) <= -1)){
+	  *dns = MINF;
+	  return;
+	}
+  }
+
+  else if (*dns != 0.0)
     return;
 
   //Stage 3: Transformation to unit Frechet
-  *dns = gev2frech(data, *nObs, *nSite, locs, scales, shapes,
-		   jac, frech);
+  if (flag)
+    *dns = gev2frechTrend(data, *nObs, *nSite, locs, scales, shapes, trendlocs, trendscales,
+			  trendshapes, jac, frech);
+
+  else
+    *dns = gev2frech(data, *nObs, *nSite, locs, scales, shapes, jac, frech);
 
   if (*dns != 0.0)
     return;
@@ -149,18 +183,28 @@ void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *
   //Stage 5: Removing the penalizing terms (if any)
   // 1- For the location parameter
   if (*locpenalty > 0)
-    *dns -= penalization(locpenmat, loccoeff, *locpenalty,
-			 *nloccoeff, *npparloc);
+    *dns -= penalization(locpenmat, loccoeff, *locpenalty, *nloccoeff, *npparloc);
   
   // 2- For the scale parameter
   if (*scalepenalty > 0)    
-    *dns -= penalization(scalepenmat, scalecoeff, *scalepenalty,
-			 *nscalecoeff, *npparscale);
+    *dns -= penalization(scalepenmat, scalecoeff, *scalepenalty, *nscalecoeff, *npparscale);
   
   // 3- For the shape parameter
   if (*shapepenalty > 0)
-    *dns -= penalization(shapepenmat, shapecoeff, *shapepenalty,
-			 *nshapecoeff, *npparshape);
+    *dns -= penalization(shapepenmat, shapecoeff, *shapepenalty, *nshapecoeff, *npparshape);
+
+  // 4- Doing the same thing for the temporal component
+  if (*temppenaltyloc > 0)
+    *dns -= penalization(temppenmatloc, tempcoeffloc, *temppenaltyloc, *ntempcoeffloc,
+			 *nppartempcoeffloc);
+
+  if (*temppenaltyscale > 0)
+    *dns -= penalization(temppenmatscale, tempcoeffscale, *temppenaltyscale, *ntempcoeffscale,
+			 *nppartempcoeffscale);
+
+  if (*temppenaltyshape > 0)
+    *dns -= penalization(temppenmatshape, tempcoeffshape, *temppenaltyshape, *ntempcoeffshape,
+			 *nppartempcoeffshape);
 
   return;
   
