@@ -14,7 +14,7 @@ void rbrowndirect(double *coord, double *bounds, int *nObs, int *nSite,
     smooth: the smooth parameter
        ans: the generated random field */
 
-  int i, j, k, neffSite, covmod = 6, lagi = 1, lagj = 1;
+  int i, j, k, neffSite, covmod = 6, lagi = 1, lagj = 1, nSimTot = 250;
   const double irange = 1 / *range, inflate = R_pow(4, 1 / *smooth) * *range;
   double one = 1, zero = 0, tmp;
 
@@ -45,9 +45,10 @@ void rbrowndirect(double *coord, double *bounds, int *nObs, int *nSite,
 
   for (i=*nObs;i--;){
     double poisson = 0;
-    int nKO = neffSite;
+    //int nKO = neffSite;
     
-    while (nKO){
+    //while (nKO){
+    for (int nSim=nSimTot; nSim--;){
       double *shift = (double *) R_alloc(*dim, sizeof(double));
       double *shiftedCoord = (double *) R_alloc(*dim * *nSite, sizeof(double));
       double *vario = (double *) R_alloc(neffSite, sizeof(double));
@@ -111,32 +112,27 @@ void rbrowndirect(double *coord, double *bounds, int *nObs, int *nSite,
       F77_CALL(dgemm)("T", "N", &neffSite, &neffSite, &neffSite, &one,
 		      v, &neffSite, u, &neffSite, &zero, covmat, &neffSite);
       
-      // Here is a loop of 2000 iterations with the same shifted origin
-      int l;
-      for (l=2000;l--;){
+      poisson += exp_rand();
+      double ipoisson = -log(poisson);//, thresh = 4 + ipoisson;
 	
-	poisson += exp_rand();
-	double ipoisson = -log(poisson), thresh = 3 + ipoisson;
+      /* We simulate one realisation of a gaussian random field with
+	 the required covariance function */
+      for (j=neffSite;j--;)
+	d[j] = norm_rand();
 	
-	/* We simulate one realisation of a gaussian random field with
-	   the required covariance function */
-	for (j=neffSite;j--;)
-	  d[j] = norm_rand();
-	
-	for (j=neffSite;j--;){
-	  double sum = 0;
-	  for (k=neffSite;k--;)
-	    sum += d[k] * covmat[j + k * neffSite];
+      for (j=neffSite;j--;){
+	double sum = 0;
+	for (k=neffSite;k--;)
+	  sum += d[k] * covmat[j + k * neffSite];
 	  
-	  gp[j] = sum;
-	}
+	gp[j] = sum;
+      }
 	
-	nKO = neffSite;
-	for (j=neffSite;j--;){
-	  ans[j * lagj + i * lagi] = fmax2(gp[j] - vario[j] + ipoisson,
-					   ans[j * lagj + i * lagi]);
-	  nKO -= (thresh <= ans[j * lagj +  i * lagi]);
-	}
+      //nKO = neffSite;
+      for (j=neffSite;j--;){
+	ans[j * lagj + i * lagi] = fmax2(gp[j] - vario[j] + ipoisson,
+					 ans[j * lagj + i * lagi]);
+	//nKO -= (thresh <= ans[j * lagj +  i * lagi]);
       }
     }
   }
@@ -144,7 +140,7 @@ void rbrowndirect(double *coord, double *bounds, int *nObs, int *nSite,
   PutRNGstate();
 
   for (i=*nObs * neffSite;i--;)
-    ans[i] = exp(ans[i]);
+    ans[i] = M_SQRT2 * exp(ans[i]);
 
   return;
 }
