@@ -263,7 +263,7 @@ blockSize: see rextremalttbm
       if (j > halfM)
 	j -= m;
       
-      dist[r] = pythag(steps[0] * i, steps[1] * j);
+      dist[r] = hypot(steps[0] * i, steps[1] * j);
     }
 
     //Computations of the covariances
